/*
 * packetmodeller/typedef.h
 *
 * Copyright (C) 2008-2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __TYPDEF_H
#define __TYPDEF_H

#include <net/ethernet.h>
#include <net/if.h>

/**
 * General parameters
 */
#define HOSTNAME_SIZE   32
#define MAX_MSG_SIZE    (1500 + sizeof(struct ether_header))

/**
 * General functions
 */
#define max(a, b) ((a) > (b) ? (a) : (b))
#define abs(a) ((a) < 0 ? (a * -1) : (a))

/**
 * Data types
 */
typedef unsigned char mac_addr [6];
typedef unsigned char host_id [13];
typedef char RAWPACKET [MAX_MSG_SIZE];
typedef char ifname [IF_NAMESIZE];
typedef char ifarray[3][IF_NAMESIZE];

/** Define MAC header (in first sight for debugging) */
struct mac_hdr {
    mac_addr mac_dest;
    mac_addr mac_src;
    unsigned char pkt_type[2];
};

/**
 *  Public functions
 */

/** Retrieve MAC addresses from Ethernet frame */
int
get_mac(mac_addr *source_mac, mac_addr *dest_mac, const RAWPACKET *packet);

/** Return IP packet type (0 = IPv4, 6 = ARP, -1 = unknown) */
int
get_packet_type(char *type_string, const RAWPACKET *packet);

#endif
