/*
 * wlanmodel/nodemanager.cc
 *
 * Copyright (C) 2008-2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "nodemanager.h"

/**
 * Constructor
 */
NodeManager::NodeManager()
{
}

/**
 * Destructor
 */
NodeManager::~NodeManager()
{
    std::map<std::string, NodeInfo*>::iterator iter = activeNodes.begin();
    while (iter != activeNodes.end())
    {
        delete iter->second;
        iter++;
    }
}

/**
 * Define participating nodes collection
 */
std::map<std::string, NodeInfo*> NodeManager::activeNodes;

/**
 * Register node at NodeManager
 */
int
NodeManager::addNode(NodeInfo *node)
{
    using namespace std;

    int size = activeNodes.size();
    activeNodes.insert( pair<string, NodeInfo*> (node->getHostId(),node) );
    if (activeNodes.size() > size)
    {
        ev << "NodeManager.addNode(): " << node->getHostId() << " added\n";
        return 0;
    } else {
        return -1;
    }
}

/**
 * Check if node is registered
 */
bool
NodeManager::isRegistered(const host_id *hostId)
{
    using namespace std;

    char *host_idp = (char *) hostId;
    string nodeId(host_idp);
    map<string, NodeInfo*>::iterator iter = activeNodes.find(nodeId);
    bool isRegistered = (iter != activeNodes.end());

    ev << "NodeManager.isRegistered(): ";
    if (isRegistered)
    {
        ev << "true\n";
    } else {
        ev << "false\n";
    }

    return isRegistered;
}

/**
 * Returns proxy gate number of given node
 */
int
NodeManager::getProxyGate(const host_id *hostId)
{
    using namespace std;
    int i;

    char *host_idp = (char *) hostId;
    string nodeId(host_idp);
    map<string, NodeInfo*>::iterator iter = activeNodes.begin();
    int numNodes = activeNodes.size();
    if (numNodes != 0)
    {
        for (i = 0; i < numNodes; i++)
        {
            if (iter->first.compare(nodeId) == 0)
            {
                int gate = (iter->second->getProxyGate());
                ev << "NodeManager.getProxyGate(): key " << nodeId << \
                    " has gate " << gate << "\n";
                return gate;
            } else {
                iter++;
            }
        }
    }
    return -1;
}

/**
 *  Returns the NodeInfo object of a given gate
 */
NodeInfo*
NodeManager::getNodeInfo(const int gateIndex)
{
    using namespace std;
    int i;

    map<string, NodeInfo*>::iterator iter = activeNodes.begin();
    int numNodes = activeNodes.size();
    if (numNodes != 0)
    {
        for (i = 0; i < numNodes; i++)
        {
            if (iter->second->getProxyGate() == gateIndex)
            {
                NodeInfo *info = iter->second;
                ev << "NodeManager.getNodeInfo: gate " << gateIndex << \
                    " belongs to " << info->getHostName() << "\n";
                return info;
            } else {
                iter++;
            }
        }
    }
    char errorMsg[70];
    sprintf(errorMsg, "NodeManager::getNodeInfo() - Couldn't find NodeInfo \
for %i\n", gateIndex);
    throw new cRuntimeError(errorMsg);
    return NULL;
}
